from django.core.exceptions import ValidationError
from django.utils.translation import ugettext_lazy as _

from authentic2.registration_backend.forms import RegistrationForm as AuthenticRegistrationForm
from authentic2 import compat


from .. import models


class RegistrationForm(AuthenticRegistrationForm):
    def __init__(self, *args, **kwargs):
        super(RegistrationForm, self).__init__(*args, **kwargs)
        insert_index = 0
        for field in models.User.REGISTER_FIELDS:
            if field in self.fields:
                impl = self.fields.pop(field)
            else:
                impl = models.User._meta.get_field(field).formfield()
            self.fields.insert(insert_index, field, impl)
            insert_index += 1

    def clean_email(self):
        """
        Verify if email is unique
        """
        User = compat.get_user_model()
        if User.objects.filter(email__iexact=self.cleaned_data['email']):
            raise ValidationError(_('This email address is already in '
                'use. Please supply a different email address.'))
        return self.cleaned_data['email']

